@@ -14,6 +14,7 @@
 #    under the License.
 import collections
 import logging
+import os
 
 import pkg_resources
 import requests
@@ -157,7 +158,8 @@ class Sushy(base.ResourceBase):
                  root_prefix='/redfish/v1/', verify=True,
                  auth=None, connector=None,
                  public_connector=None,
-                 language='en'):
+                 language='en', server_side_retries=10,
+                 server_side_retries_delay=3):
         """A class representing a RootService
 
         :param base_url: The base URL to the Redfish controller. It
@@ -180,6 +182,10 @@ class Sushy(base.ResourceBase):
             on the Internet, e.g., for Message Registries. Defaults to None.
         :param language: RFC 5646 language code for Message Registries.
             Defaults to 'en'.
+        :param server_side_retries: Number of times to retry GET requests in
+            case of server side errors. Defaults to 10.
+        :param server_side_retries_delay: Time in seconds between retries of
+            GET requests in case of server side errors. Defaults to 3.
         """
         self._root_prefix = root_prefix
         if (auth is not None and (password is not None
@@ -193,7 +199,10 @@ class Sushy(base.ResourceBase):
         self._auth = auth
 
         super(Sushy, self).__init__(
-            connector or sushy_connector.Connector(base_url, verify=verify),
+            connector or sushy_connector.Connector(
+                base_url, verify=verify,
+                server_side_retries=server_side_retries,
+                server_side_retries_delay=server_side_retries_delay),
             path=self._root_prefix)
         self._public_connector = public_connector or requests
         self._language = language
@@ -380,10 +389,15 @@ class Sushy(base.ResourceBase):
 
     def get_sessions_path(self):
         """Returns the Sessions url"""
-
+        # NOTE(TheJulia): This method is the standard discovery method
+        # advised by the DMTF DSP0266 standard to find the session service.
         try:
             links_url = self.json.get('Links')
-            return links_url['Sessions']['@odata.id']
+            sessions_uri = links_url['Sessions']['@odata.id']
+            # Save the session URL for post detection and prevention
+            # of recursive autentication attempts.
+            self._conn._sessions_uri = sessions_uri
+            return sessions_uri
         except (TypeError, KeyError):
             raise exceptions.MissingAttributeError(
                 attribute='Links/Sessions/@data.id', resource=self.path)
@@ -399,6 +413,62 @@ class Sushy(base.ResourceBase):
             redfish_version=self.redfish_version,
             registries=self.lazy_registries, root=self)
 
+    def create_session(self, username=None, password=None):
+        """Creates a session without invoking SessionService.
+
+        For use when a new connection is to be established. Removes prior
+        Session and authentication data before making the request.
+
+        :param username: The username to utilize to create a session with the
+                         remote endpoint.
+        :param password: The password to utilize to create a session with the
+                         remote endpoint.
+        :returns: A session key and uri in the form of a tuple
+        :raises: MissingXAuthToken
+        :raises: ConnectionError
+        :raises: AccessError
+        :raises: HTTPError
+        :raises: MissingAttributeError
+        """
+        # Explicitly removes in-client session data to proceed. This prevents
+        # AccessErrors as prior authentication shouldn't be submitted with a
+        # new authentication attempt, and doing so with old/invalid session
+        # data can result in an AccessError being raised.
+        self._conn._session.auth = None
+        if 'X-Auth-Token' in self._conn._session.headers:
+            # Delete the token value that was saved to the session
+            # as otherwise we would end up with a dictionary containing
+            # a {'X-Auth-Token': null} being sent across to the remote
+            # bmc.
+            del self._conn._session.headers['X-Auth-Token']
+        try:
+            session_service_path = self.get_sessions_path()
+        except (exceptions.MissingAttributeError, exceptions.AccessError):
+            # Guesses path based upon DMTF standard, in the event
+            # the links resource on the service root is incorrect.
+            session_service_path = os.path.join(self._path,
+                                                'SessionService/Sessions')
+            LOG.warning('Could not discover the Session service path, '
+                        'falling back to %s.',
+                        session_service_path)
+            session_url = self._root_prefix + 'SessionService/Sessions'
+            self._conn._sessions_uri = session_url
+
+        data = {'UserName': username, 'Password': password}
+        LOG.debug("Requesting new session from %s.",
+                  session_service_path)
+        rsp = self._conn.post(session_service_path, data=data)
+        session_key = rsp.headers.get('X-Auth-Token')
+        if session_key is None:
+            raise exceptions.MissingXAuthToken(
+                method='POST', url=session_service_path, response=rsp)
+
+        session_uri = rsp.headers.get('Location')
+        if session_uri is None:
+            LOG.warning("Received X-Auth-Token but NO session uri.")
+
+        return session_key, session_uri
+
     def get_update_service(self):
         """Get the UpdateService object
 
@@ -527,6 +597,7 @@ class Sushy(base.ResourceBase):
 
         registry_col = self._get_registry_collection()
 
+        endpoint_registries = {}
         if registry_col:
             provided = registry_col.get_members()
             for r in provided:
@@ -540,9 +611,16 @@ class Sushy(base.ResourceBase):
                         self._language,
                         self._public_connector)
                 if registry:
-                    registries[r.registry] = registry
-                    if r.identity not in registries:
-                        registries[r.identity] = registry
+                    endpoint_registries[r.registry] = registry
+                    endpoint_registries.setdefault(r.identity, registry)
+
+        if endpoint_registries:
+            LOG.debug('Found registries for %(id)s: %(reg)s',
+                      {'id': self.identity,
+                       'reg': ', '.join(endpoint_registries)})
+            registries.update(endpoint_registries)
+        else:
+            LOG.debug('No registries are available for %s', self.identity)
 
         return registries
 
