@@ -19,12 +19,15 @@
 import collections
 import logging
 
+from dateutil import parser
+
 from sushy import exceptions
 from sushy.resources import base
 from sushy.resources.chassis import chassis
 from sushy.resources import common
 from sushy.resources import constants as res_cons
 from sushy.resources.manager import manager
+from sushy.resources.manager import virtual_media
 from sushy.resources import settings
 from sushy.resources.system import bios
 from sushy.resources.system import constants as sys_cons
@@ -56,6 +59,25 @@ class BootField(base.CompositeField):
 
     target = base.MappedField('BootSourceOverrideTarget', sys_cons.BootSource)
 
+    http_boot_uri = base.Field('HttpBootUri')
+
+
+class BootProgressField(base.CompositeField):
+
+    last_boot_seconds_count = base.Field('LastBootTimeSeconds',
+                                         adapter=utils.int_or_none)
+    """The number of seconds the last boot took to reach OSRunning."""
+
+    last_state = base.MappedField('LastState', sys_cons.BootProgressStates)
+    """The last recorded boot progress states."""
+
+    last_state_updated_at = base.Field('LastStateTime',
+                                       adapter=parser.parse)
+    """The date-time value when the last state field was updated."""
+
+    oem_last_state = base.Field('OemLastState')
+    """The OEM last state time to describe OEM specific state information."""
+
 
 class MemorySummaryField(base.CompositeField):
     health = base.Field(['Status', 'HealthRollup'])
@@ -138,6 +160,9 @@ class System(base.ResourceBase):
 
     _actions = ActionsField('Actions', required=True)
 
+    boot_progress = BootProgressField('BootProgress')
+    """The last updated boot progress indicator"""
+
     def __init__(self, connector, identity, redfish_version=None,
                  registries=None, root=None):
         """A class representing a ComputerSystem
@@ -212,7 +237,8 @@ class System(base.ResourceBase):
         return {v for v in sys_cons.BootSource
                 if v.value in self.boot.allowed_values}
 
-    def set_system_boot_options(self, target=None, enabled=None, mode=None):
+    def set_system_boot_options(self, target=None, enabled=None, mode=None,
+                                http_boot_uri=None):
         """Set boot source and/or boot frequency and/or boot mode.
 
         Set the boot source and/or boot frequency and/or boot mode to use
@@ -220,14 +246,31 @@ class System(base.ResourceBase):
 
         :param target: The target boot source,
             a :py:class:`sushy.BootSource` value. Optional.
-        :param enabled: How long the override be enabled,
+        :param enabled: How long the override is enabled,
             a :py:class:`sushy.BootSourceOverrideEnabled` value. Optional.
         :param mode: The boot mode,
             a :py:class:`sushy.BootSourceOverrideMode` value. Optional.
+        :param http_boot_uri: The requested HTTP Boot URI to transmit to the
+            BMC. Only valid when BootSourceOverrideTarget is set to UefiHTTP,
+            when utilizing the ``target`` parameter. If no value is supplied,
+            and the target is set to UefiHTTP, then an empty value will be
+            sent to the BMC to remove any prior setting, allowing the host
+            to load configuration from DHCP.
+            If not explicitly set, any value will be removed from a BMC when
+            UefiHttp boot is not engaged.
+
         :raises: InvalidParameterValueError, if any information passed is
             invalid.
         """
         data = collections.defaultdict(dict)
+        settings_data = collections.defaultdict(dict)
+
+        if self._settings and self._settings.resource_uri:
+            settings_resp = self._conn.get(self._settings.resource_uri)
+            settings_boot_section = settings_resp.json().get('Boot', {})
+        else:
+            settings_resp = None
+            settings_boot_section = {}
 
         if target is not None:
             valid_targets = self.get_allowed_system_boot_source_values()
@@ -253,8 +296,12 @@ class System(base.ResourceBase):
                           'machine. Overriding boot device from %s to %s.',
                           target, sys_cons.BootSource.USB_CD)
                 target = sys_cons.BootSource.USB_CD
-
-            data['Boot']['BootSourceOverrideTarget'] = target.value
+            if (settings_resp and "BootSourceOverrideTarget" in
+                    settings_boot_section):
+                settings_data['Boot']['BootSourceOverrideTarget'] = \
+                    target.value
+            else:
+                data['Boot']['BootSourceOverrideTarget'] = target.value
 
         if enabled is not None:
             try:
@@ -263,8 +310,11 @@ class System(base.ResourceBase):
                 raise exceptions.InvalidParameterValueError(
                     parameter='enabled', value=enabled,
                     valid_values=list(sys_cons.BootSourceOverrideEnabled))
-
-            data['Boot']['BootSourceOverrideEnabled'] = fishy_freq
+            if (settings_resp and "BootSourceOverrideEnabled" in
+                    settings_boot_section):
+                settings_data['Boot']['BootSourceOverrideEnabled'] = fishy_freq
+            else:
+                data['Boot']['BootSourceOverrideEnabled'] = fishy_freq
 
         if mode is not None:
             try:
@@ -273,26 +323,42 @@ class System(base.ResourceBase):
                 raise exceptions.InvalidParameterValueError(
                     parameter='mode', value=mode,
                     valid_values=list(sys_cons.BootSourceOverrideMode))
+            if (settings_resp and "BootSourceOverrideMode" in
+                    settings_boot_section):
+                settings_data['Boot']['BootSourceOverrideMode'] = fishy_mode
+            else:
+                data['Boot']['BootSourceOverrideMode'] = fishy_mode
 
-            data['Boot']['BootSourceOverrideMode'] = fishy_mode
-
-        headers = None
-        etag = self._get_etag()
-        path = self.path
-
-        # NOTE(janders): checking if @Redfish.Settings are available; if so
-        # using the URI from the SettingsObject. Also ensuring the ETag URI
-        # matches in this case (the one above would not be valid for new URI).
-        if self._settings and self._settings.resource_uri:
-            path = self._settings.resource_uri
-            resp = self._conn.get(path)
-            etag = resp.headers.get('ETag')
+        if target == sys_cons.BootSource.UEFI_HTTP:
+            # The http_boot_uri value *can* be set independently of the
+            # target, but the BMC will just ignore it unless the target
+            # is set. So we should only, and explicitly set it when we've
+            # been requested to boot from UefiHTTP.
+            if not http_boot_uri:
+                # This should clear out any old entries, as no URI translates
+                # to the intent of "use whatever the dhcp server says".
+                data['Boot']['HttpBootUri'] = None
+            else:
+                # Explicilty set the URI.
+                data['Boot']['HttpBootUri'] = http_boot_uri
+        elif not http_boot_uri:
+            # We're not doing boot from URL, we should cleanup any setting
+            # which may be from a prior step/call.
+            if settings_boot_section.get('HttpBootUri'):
+                # If the setting is present, and has any value, unset it.
+                data['Boot']['HttpBootUri'] = None
 
         # TODO(lucasagomes): Check the return code and response body ?
         #                    Probably we should call refresh() as well.
-        if etag is not None:
-            headers = {'If-Match': etag}
-        self._conn.patch(path, data=data, headers=headers)
+        if settings_data.get('Boot'):
+            etag = settings_resp.headers.get('ETag')
+            path = self._settings.resource_uri
+            self._conn.patch(path, data=settings_data,
+                             etag=etag)
+        if data.get('Boot'):
+            etag = self._get_etag()
+            path = self.path
+            self._conn.patch(path, data=data, etag=etag)
 
     # TODO(etingof): we should remove this method, eventually
     def set_system_boot_source(
@@ -331,9 +397,10 @@ class System(base.ResourceBase):
                 parameter='state', value=state,
                 valid_values=' ,'.join(i.value for i in res_cons.IndicatorLED))
 
+        etag = self._get_etag()
         data = {'IndicatorLED': state}
 
-        self._conn.patch(self.path, data=data)
+        self._conn.patch(self.path, data=data, etag=etag)
         self.invalidate()
 
     def _get_processor_collection_path(self):
@@ -482,6 +549,18 @@ class System(base.ResourceBase):
             redfish_version=self.redfish_version,
             registries=self.registries, root=self.root)
 
+    @property
+    @utils.cache_it
+    def virtual_media(self):
+        """Property to reference `VirtualMedia` instance
+
+        :returns: A `VirtualMediaCollection` instance.
+        """
+        return virtual_media.VirtualMediaCollection(
+            self._conn, utils.get_sub_resource_path_by(self, 'VirtualMedia'),
+            redfish_version=self.redfish_version, registries=self.registries,
+            root=self.root)
+
 
 class SystemCollection(base.ResourceCollectionBase):
 
