@@ -23,6 +23,7 @@ from sushy import exceptions
 from sushy.resources.chassis import chassis
 from sushy.resources import constants as res_cons
 from sushy.resources.manager import manager
+from sushy.resources.manager import virtual_media
 from sushy.resources.oem import fake
 from sushy.resources.system import bios
 from sushy.resources.system import processor
@@ -46,6 +47,8 @@ class SystemTestCase(base.TestCase):
         self.sys_inst = system.System(
             self.conn, '/redfish/v1/Systems/437XR1138R2',
             redfish_version='1.0.2')
+        self.sys_inst._get_etag = mock.Mock()
+        self.sys_inst._get_etag.return_value = '81802dbf61beb0bd'
 
     def test__parse_attributes(self):
         self.sys_inst._parse_attributes(self.json_doc)
@@ -80,6 +83,14 @@ class SystemTestCase(base.TestCase):
                          .maintenance_window_start_time)
         for oem_vendor in self.sys_inst.oem_vendors:
             self.assertIn(oem_vendor, ('Contoso', 'Chipwise'))
+        self.assertEqual(sushy.BootProgressStates.OS_RUNNING,
+                         self.sys_inst.boot_progress.last_state)
+        self.assertEqual(66,
+                         self.sys_inst.boot_progress.last_boot_seconds_count)
+        self.assertEqual(parser.parse('2017-05-03T23:12:37-05:00'),
+                         self.sys_inst.boot_progress.last_state_updated_at)
+        self.assertEqual("OS foo running.",
+                         self.sys_inst.boot_progress.oem_last_state)
 
     def test__parse_attributes_return(self):
         attributes = self.sys_inst._parse_attributes(self.json_doc)
@@ -243,7 +254,8 @@ class SystemTestCase(base.TestCase):
                         sushy.BootSource.UTILITIES,
                         sushy.BootSource.DIAGS,
                         sushy.BootSource.SD_CARD,
-                        sushy.BootSource.UEFI_TARGET])
+                        sushy.BootSource.UEFI_TARGET,
+                        sushy.BootSource.UEFI_HTTP])
         self.assertEqual(expected, values)
         self.assertIsInstance(values, set)
 
@@ -283,7 +295,7 @@ class SystemTestCase(base.TestCase):
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideTarget': 'Pxe',
                            'BootSourceOverrideMode': 'UEFI'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_no_mode_specified(self):
         self.sys_inst.set_system_boot_options(
@@ -293,7 +305,7 @@ class SystemTestCase(base.TestCase):
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
                            'BootSourceOverrideTarget': 'Hdd'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_no_target_specified(self):
         self.sys_inst.set_system_boot_options(
@@ -303,7 +315,7 @@ class SystemTestCase(base.TestCase):
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideMode': 'UEFI'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_no_freq_specified(self):
         self.sys_inst.set_system_boot_options(
@@ -313,7 +325,7 @@ class SystemTestCase(base.TestCase):
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideTarget': 'Pxe',
                            'BootSourceOverrideMode': 'UEFI'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_nothing_specified(self):
         self.sys_inst.set_system_boot_options()
@@ -348,7 +360,7 @@ class SystemTestCase(base.TestCase):
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
                            'BootSourceOverrideTarget': 'UsbCd'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_supermicro_no_usb_cd_boot(self):
 
@@ -361,7 +373,7 @@ class SystemTestCase(base.TestCase):
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
                            'BootSourceOverrideTarget': 'Cd'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_settings_resource_nokia(self):
         with open('sushy/tests/unit/json_samples/settings-nokia.json') as f:
@@ -477,6 +489,57 @@ class SystemTestCase(base.TestCase):
             data={'Boot': {'BootSourceOverrideMode': 'UEFI'}},
             etag='"3d7b838291941d"')
 
+    def test_set_system_boot_options_httpbooturi(self):
+        self.sys_inst.set_system_boot_options(
+            sushy.BootSource.UEFI_HTTP,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE,
+            mode=sushy.BootSourceOverrideMode.UEFI,
+            http_boot_uri='http://test.lan/test_image.iso'
+            )
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideTarget': 'UefiHttp',
+                           'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideMode': 'UEFI',
+                           'HttpBootUri': 'http://test.lan/test_image.iso'}},
+            etag=mock.ANY)
+
+    def test_set_system_boot_options_httpboot(self):
+        self.sys_inst.set_system_boot_options(
+            sushy.BootSource.UEFI_HTTP,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE,
+            mode=sushy.BootSourceOverrideMode.UEFI
+            )
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideTarget': 'UefiHttp',
+                           'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideMode': 'UEFI',
+                           'HttpBootUri': None}},
+            etag=mock.ANY)
+
+    def test_set_system_boot_options_httpboot_unset(self):
+        self.sys_inst._settings = mock.Mock()
+        self.sys_inst._settings.resource_uri = 'meow'
+        settings_body = json.dumps(
+            {'Boot': {'HttpBootUri': 'http://foo.bar'}}
+        )
+
+        get_settings = mock.MagicMock()
+        get_settings.json.return_value = settings_body
+        self.conn.get.side_effect = get_settings
+
+        self.sys_inst.set_system_boot_options(
+            sushy.BootSource.HDD,
+            mode=sushy.BootSourceOverrideMode.UEFI
+            )
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideTarget': 'Hdd',
+                           'BootSourceOverrideMode': 'UEFI',
+                           'HttpBootUri': None}},
+            etag=mock.ANY)
+
     def test_set_system_boot_source(self):
         self.sys_inst.set_system_boot_source(
             sushy.BootSource.PXE,
@@ -487,10 +550,10 @@ class SystemTestCase(base.TestCase):
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideTarget': 'Pxe',
                            'BootSourceOverrideMode': 'UEFI'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_source_with_etag(self):
-        self.conn.get.return_value.headers = {'ETag': '"3d7b838291941d"'}
+        self.conn.get.return_value.headers = {'ETag': '"81802dbf61beb0bd"'}
         self.sys_inst.set_system_boot_source(
             sushy.BOOT_SOURCE_TARGET_PXE,
             enabled=sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
@@ -500,7 +563,7 @@ class SystemTestCase(base.TestCase):
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideTarget': 'Pxe',
                            'BootSourceOverrideMode': 'UEFI'}},
-            etag='"3d7b838291941d"')
+            etag="81802dbf61beb0bd")
 
     def test_set_system_boot_source_no_mode_specified(self):
         self.sys_inst.set_system_boot_source(
@@ -510,7 +573,17 @@ class SystemTestCase(base.TestCase):
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
                            'BootSourceOverrideTarget': 'Hdd'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
+
+    def test_set_system_boot_unsets_http_boot_uri(self):
+        self.sys_inst.set_system_boot_source(
+            sushy.BootSource.HDD,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE)
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'Hdd'}},
+            etag=mock.ANY)
 
     def test_set_system_boot_source_invalid_target(self):
         self.assertRaises(exceptions.InvalidParameterValueError,
@@ -533,7 +606,8 @@ class SystemTestCase(base.TestCase):
             self.sys_inst.set_indicator_led(sushy.IndicatorLED.BLINKING)
             self.sys_inst._conn.patch.assert_called_once_with(
                 '/redfish/v1/Systems/437XR1138R2',
-                data={'IndicatorLED': 'Blinking'})
+                data={'IndicatorLED': 'Blinking'},
+                etag='81802dbf61beb0bd')
 
             invalidate_mock.assert_called_once_with()
 
@@ -837,6 +911,85 @@ class SystemTestCase(base.TestCase):
         self.assertIs(self.sys_inst, contoso_system_extn_inst._parent_resource)
         self.assertEqual('Contoso', contoso_system_extn_inst._vendor_id)
 
+    def test_no_virtual_media_attr(self):
+        with self.assertRaisesRegex(
+            exceptions.MissingAttributeError, 'attribute VirtualMedia'):
+            self.sys_inst.virtual_media
+
+
+class SystemWithVirtualMedia(base.TestCase):
+
+    def setUp(self):
+        super(SystemWithVirtualMedia, self).setUp()
+        self.conn = mock.Mock()
+        with open('sushy/tests/unit/json_samples/'
+                  'systemv1_20.json') as f:
+            self.json_doc = json.load(f)
+
+        self.conn.get.return_value.json.return_value = self.json_doc
+
+        self.sys_inst = system.System(
+            self.conn, '/redfish/v1/Systems/437XR1138R2',
+            redfish_version='1.0.2')
+
+    def test_virtual_media(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_mediav1_6.json') as f:
+            virtual_media_json = json.load(f)
+
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_media_collectionv1_6.json') as f:
+            virtual_media_collection_json = json.load(f)
+
+        self.conn.get.return_value.json.side_effect = [
+            virtual_media_collection_json, virtual_media_json]
+
+        # | WHEN |
+        actual_virtual_media = self.sys_inst.virtual_media
+
+        # | THEN |
+        self.assertIsInstance(actual_virtual_media,
+                              virtual_media.VirtualMediaCollection)
+        self.assertEqual(actual_virtual_media.name, 'Virtual Media Services')
+
+        member = actual_virtual_media.get_member(
+            '/redfish/v1/Systems/437XR1138R2/VirtualMedia/CD1')
+
+        self.assertEqual(member.image_name, 'mymedia-read-only')
+        self.assertTrue(member.inserted)
+        self.assertFalse(member.write_protected)
+        self.assertTrue(member.verify_certificate)
+
+    def test_virtual_media_on_refresh(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_media_collectionv1_6.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        # | WHEN & THEN |
+        actual_virtual_media = self.sys_inst.virtual_media
+        self.assertIsInstance(actual_virtual_media,
+                              virtual_media.VirtualMediaCollection)
+
+        with open('sushy/tests/unit/json_samples/systemv1_20.json', 'r') as f:
+            self.conn.get.return_value.json.return_value = json.loads(f.read())
+
+        self.sys_inst.invalidate()
+        self.sys_inst.refresh(force=False)
+
+        self.assertTrue(actual_virtual_media._is_stale)
+
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_media_collectionv1_6.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.virtual_media,
+                              virtual_media.VirtualMediaCollection)
+        self.assertFalse(actual_virtual_media._is_stale)
+
 
 class SystemCollectionTestCase(base.TestCase):
 
